@@ -106,6 +106,8 @@ public:
   /// either  fDBAPIModule_Data or fDBAPIModule_Admin to select plugin handling of
   /// either data or admin independently.
   bool fDataModuleAlsoHandlesAdmin;
+  // wants startDataRead() called as early as possible
+  bool fEarlyStartDataRead;
   // - config object for API module
   TDB_Api_Config fDBApiConfig_Data;
   TDB_Api_Config fDBApiConfig_Admin;
@@ -170,6 +172,10 @@ public:
   virtual void dsResetDataStore(void) { InternalResetDataStore(); inherited::dsResetDataStore(); };
   virtual ~TPluginApiDS();
 
+  // override TSyncDataStore: the plugin must be able to return 404
+  // when an item is not found during delete
+  virtual bool dsDeleteDetectsItemPresence() const { return true; }
+
   #ifndef BINFILE_ALWAYS_ACTIVE
   /// @name apiXXXX methods defining the interface from TCustomImplDS to TXXXApi actual API implementations
   /// @{
@@ -196,7 +202,7 @@ public:
   ///   - fPreviousToRemoteSyncIdentifier = string identifying last session that sent data to remote
   ///                         (or last changelog update in case of BASED_ON_BINFILE_CLIENT). Needs only be saved
   ///                         if derived datastore cannot work with timestamps and has its own identifier.
-	///   - fMapTable         = list<TMapEntry> containing map entries. The implementation must load all map entries
+  ///   - fMapTable         = list<TMapEntry> containing map entries. The implementation must load all map entries
   ///                         related to the current sync target identified by the triple of (aDeviceID,aDatabaseID,aRemoteDBID)
   ///                         or by fTargetKey. The entries added to fMapTable must have "changed", "added" and "deleted" flags
   ///                         set to false.
@@ -252,6 +258,9 @@ public:
   ///   them separately afterwards).
   #endif // not BINFILE_ALWAYS_ACTIVE
 
+  /// perform early data access start (if datastore requests it by setting fEarlyStartDataRead config flag)
+  virtual localstatus apiEarlyDataAccessStart(void);
+  /// read the sync set
   virtual localstatus apiReadSyncSet(bool aNeedAll);
   /// Zap all data in syncset (note that everything outside the sync set will remain intact)
   virtual localstatus apiZapSyncSet(void);
@@ -314,6 +323,8 @@ public:
 private:
   // - connect data handling part of plugin. Returns LOCERR_NOTIMPL when no data plugin is selected
   TSyError connectDataPlugin(void);
+  // - prepare for reading syncset (is called early when fEarlyStartDataRead is set, otherwise from within apiReadSyncSet)
+  localstatus apiPrepareReadSyncSet(void);
   // - alert possible thread change to plugins
   //   Does not check if API is locked or not, see dsThreadMayChangeNow()
   void ThreadMayChangeNow(void);
@@ -369,8 +380,8 @@ private:
   // config pointer
   TPluginDSConfig *fPluginDSConfigP;
   // filter testing
-	bool fAPICanFilter;
-	bool fAPIFiltersTested;  
+  bool fAPICanFilter;
+  bool fAPIFiltersTested;
 }; // TPluginApiDS
 
 
@@ -409,6 +420,6 @@ private:
 
 } // namespace sysync
 
-#endif	// PLUGINAPIDS_H
+#endif  // PLUGINAPIDS_H
 
 // eof
