@@ -1,7 +1,7 @@
 /*
  *  File:         SyncAgent.h
  *
- *  Author:			  Lukas Zeller (luz@plan44.ch)
+ *  Author:       Lukas Zeller (luz@plan44.ch)
  *
  *  TSyncAgent:   Provides functionality to run client or server
  *                sessions.
@@ -60,7 +60,7 @@ typedef enum {
   ses_needdata,     ///< need SyncML request data, waiting for STEPCMD_GOTDATA
   ses_processing,   ///< ready to perform next STEPCMD_STEP to process SyncML messages
   ses_generating,   ///< ready to perform next STEPCMD_STEP to generate SyncML messages
-  ses_dataready,    ///< data is ready to be sent, waiting for STEPCMD_SENTDATA  
+  ses_dataready,    ///< data is ready to be sent, waiting for STEPCMD_SENTDATA
   ses_done,         ///< session done
   numServerEngineStates
 } TServerEngineState;
@@ -71,7 +71,7 @@ typedef enum {
   ces_idle,         ///< client engine is idle and can be initialized with STEPCMD_CLIENTSTART
   ces_generating,   ///< ready to perform next STEPCMD_STEP to generate SyncML messages
   ces_dataready,    ///< data is ready to be sent, waiting for STEPCMD_SENTDATA
-	ces_resending,		///< data already sent, but app indicates with STEPCMD_RESENDDATA that it needed to resend the data
+  ces_resending,    ///< data already sent, but app indicates with STEPCMD_RESENDDATA that it needed to resend the data
   ces_needdata,     ///< need response data, waiting for STEPCMD_GOTDATA
   ces_processing,   ///< ready to perform next STEPCMD_STEP to process SyncML messages
   ces_done,         ///< session done
@@ -153,6 +153,12 @@ public:
   TAuthTypes fAssumedServerAuth;
   TFmtTypes fAssumedServerAuthEnc; // start with char encoding
   string fAssumedNonce; // start with no nonce
+  bool fPreferSlowSync; // Ask server for slow sync instead of refresh-from-server:
+                        // both have the same effect, but have different pros and cons.
+                        // Defaults to true, the traditional behavior of libsynthesis.
+                        // Some old servers only support slow sync. Some new ones
+                        // work better with refresh, because they throttle clients
+                        // which do slow syncs excessively.
   // auth retry options (mainly for stupid servers like SCTS)
   bool fNewSessionForAuthRetry; // restart session for auth retries
   bool fNoRespURIForAuthRetry; // send retry to original URI for auth retries
@@ -203,7 +209,7 @@ public:
   uInt16 fMaxGUIDSizeSent;
   // - respURI
   bool fUseRespURI;
-  bool fRespURIOnlyWhenDifferent;  
+  bool fRespURIOnlyWhenDifferent;
   #endif // SYSYNC_SERVER
   // check config elements
   #ifndef HARDCODED_CONFIG
@@ -211,7 +217,7 @@ public:
   #endif
   virtual void clear();
   virtual void localResolve(bool aLastPass);
-	#ifdef SYSYNC_SERVER
+  #ifdef SYSYNC_SERVER
   // create appropriate session (=agent) for this server
   virtual TSyncAgent *CreateServerSession(TSyncSessionHandle *aSessionHandle, const char *aSessionID)=0;
   #endif
@@ -234,7 +240,7 @@ public:
     TSyncAppBase *aAppBaseP,
     TSyncSessionHandle *aSessionHandleP,
     cAppCharP aSessionID // a session ID
-  );  
+  );
   virtual ~TSyncAgent();
   virtual void TerminateSession(void); // Terminate session, like destructor, but without actually destructing object itself
   virtual void ResetSession(void); // Resets session (but unlike TerminateSession, session might be re-used)
@@ -250,17 +256,17 @@ public:
   /// @param aInfoP[in] pointer to a TEngineProgressInfo structure, NULL if no progress info needed
   /// @return LOCERR_OK on success, SyncML or LOCERR_xxx error code on failure
   TSyError SessionStep(uInt16 &aStepCmd, TEngineProgressInfo *aInfoP);
-	#ifdef PROGRESS_EVENTS
-  // Handle (or dispatch) Session level progress event  
-	virtual bool HandleSessionProgressEvent(TEngineProgressInfo aProgressInfo);
-	#endif // PROGRESS_EVENTS
+  #ifdef PROGRESS_EVENTS
+  // Handle (or dispatch) Session level progress event
+  virtual bool HandleSessionProgressEvent(TEngineProgressInfo aProgressInfo);
+  #endif // PROGRESS_EVENTS
   #endif // ENGINE_LIBRARY
   /// @brief Get new session key to access details of this session
   virtual appPointer newSessionKey(TEngineInterface *aEngineInterfaceP);
   #endif // ENGINEINTERFACE_SUPPORT
 
-	// Server & Client common:
-	// - message start and end
+  // Server & Client common:
+  // - message start and end
   virtual bool MessageStarted(SmlSyncHdrPtr_t aContentP, TStatusCommand &aStatusCommand, bool aBad=false);
   virtual void MessageEnded(bool aIncomingFinal);
   // - handle status received for SyncHdr, returns false if not handled
@@ -281,7 +287,7 @@ public:
   virtual string getDeviceID(void);
 
 
-	#ifdef SYSYNC_CLIENT
+  #ifdef SYSYNC_CLIENT
   #ifdef ENGINE_LIBRARY
   // set profileID to client session before doing first SessionStep
   virtual void SetProfileSelector(uInt32 aProfileSelector) { fProfileSelectorInternal = aProfileSelector; /* default is just passing it on */ };
@@ -355,8 +361,8 @@ public:
   // info about server status
   virtual bool serverBusy(void); // return busy status (set by connection limit or app expiry)
   #endif // SYSYNC_SERVER
-    
-	#ifdef SYSYNC_CLIENT
+
+  #ifdef SYSYNC_CLIENT
   // variables
 protected:
   // - socks and proxy hosts if any
@@ -411,7 +417,7 @@ protected:
   uInt32 fOutgoingAlert222Count;
   // Loop detecting time frame to avoid wrong detection of "keep-alive" message
   lineartime_t fLastOutgoingAlert222;
-	// - called when essential status was received resets clien side Alert 222 loop detector
+  // - called when essential status was received resets clien side Alert 222 loop detector
   //   because receiving essential status means real session progress
   virtual void essentialStatusReceived(void) { fOutgoingAlert222Count = 0; };
 public:
@@ -420,6 +426,10 @@ public:
   bool fServerHasSINCEBEFORE;
   #endif // SYSYNC_CLIENT
 
+  // flag controlled by app or datastore via "restartsync" session
+  // variable: if true, the agent will try to restart the sync and
+  // then clear the flag
+  bool fRestartSyncOnce;
   #ifdef SYSYNC_SERVER
 public:
   // respuri
@@ -444,6 +454,9 @@ protected:
   virtual void getNextNonce(const char *aDeviceID, string &aNextNonce);
   // - get nonce string, which is expected to be used by remote party for MD5 auth.
   virtual void getAuthNonce(const char *aDeviceID, string &aAuthNonce);
+  // restarting a sync session is allowed in non-standard mode
+  // TODO: introduce DevInf extensions and use that instead
+  virtual bool allowAlertAfterMap() { return fCompleteFromClientOnly; }
   #ifdef ENGINE_LIBRARY
   // Engine interface
   // - process step
@@ -471,7 +484,7 @@ protected:
   uInt32 fIncomingBytes;
   uInt32 fOutgoingBytes;
   // server session handle
-  TSyncSessionHandle *fSessionHandleP; // the session "handle" (wrapper, containing server specific locking etc.)  
+  TSyncSessionHandle *fSessionHandleP; // the session "handle" (wrapper, containing server specific locking etc.)
   #endif // SYSYNC_SERVER
 
   #ifdef ENGINE_LIBRARY
@@ -492,7 +505,21 @@ protected:
   bool fSuspendRequested;
   bool fAbortRequested;
   #endif // ENGINE_LIBRARY
-	
+
+  // instead of sending <Final> at the end of the last message,
+  // the agent is going to ask the server for another sync session;
+  // set in NextMessage() for restarting in ClientMessageEnded()
+  bool fRestartingSync;
+
+  // true if the SyncML client shall continue the session
+  // by sending another package with <Alert>, instead of closing
+  // the session
+  bool restartSync();
+
+  // true if all active stores are in "from client only" mode and
+  // the engine is not in the fCompleteFromClientOnly mode
+  // => map phase can be skipped
+  bool checkAllFromClientOnly();
 }; // TSyncAgent
 
 
@@ -552,8 +579,8 @@ public:
 
 #endif // ENGINEINTERFACE_SUPPORT
 
-}	// namespace sysync
+} // namespace sysync
 
-#endif	// SYNC_AGENT_H
+#endif  // SYNC_AGENT_H
 
 // eof
